@@ -9,7 +9,7 @@ use strict;
 use warnings;
 no warnings 'recursion'; # Disable the "deep recursion" warning
 
-our $VERSION = '0.31';
+our $VERSION = '0.32';
 
 use Carp qw(); # don't import croak
 use Scalar::Util qw( weaken blessed reftype );
@@ -1403,7 +1403,7 @@ sub wait_all
    my @subs = @_;
 
    unless( @subs ) {
-      my $self = Future->done;
+      my $self = $class->done;
       $self->{subs} = [];
       return $self;
    }
@@ -1461,7 +1461,7 @@ sub wait_any
    my @subs = @_;
 
    unless( @subs ) {
-      my $self = Future->fail( "Cannot ->wait_any with no subfutures" );
+      my $self = $class->fail( "Cannot ->wait_any with no subfutures" );
       $self->{subs} = [];
       return $self;
    }
@@ -1551,7 +1551,7 @@ sub needs_all
    my @subs = @_;
 
    unless( @subs ) {
-      my $self = Future->done;
+      my $self = $class->done;
       $self->{subs} = [];
       return $self;
    }
@@ -1651,7 +1651,7 @@ sub needs_any
    my @subs = @_;
 
    unless( @subs ) {
-      my $self = Future->fail( "Cannot ->needs_any with no subfutures" );
+      my $self = $class->fail( "Cannot ->needs_any with no subfutures" );
       $self->{subs} = [];
       return $self;
    }
@@ -1874,6 +1874,13 @@ callback. C<$operation_name> is a string giving the reason for which the
 callback is being saved; currently one of C<on_ready>, C<on_done>, C<on_fail>
 or C<sequence>; the latter being used for all the sequence-returning methods.
 
+This method is intentionally invoked only for CODE references that are being
+saved on a pending C<Future> instance to be invoked at some later point. It
+does not run for callbacks to be invoked on an already-complete instance. This
+is for performance reasons, where the intended behaviour is that the wrapper
+can provide some amount of context save and restore, to return the operating
+environment for the callback back to what it was at the time it was saved.
+
 For example, the following wrapper saves the value of a package variable at
 the time the callback was saved, and restores that value at invocation time
 later on. This could be useful for preserving context during logging in a
@@ -1881,8 +1888,10 @@ Future-based program.
 
  our $LOGGING_CTX;
 
- around "Future::wrap_cb" => sub {
-    my $orig = shift;
+ no warnings 'redefine';
+
+ my $orig = Future->can( "wrap_cb" );
+ *Future::wrap_cb = sub {
     my $cb = $orig->( @_ );
 
     my $saved_logging_ctx = $LOGGING_CTX;
